/*
  myTemp is used to connect PalmOS PDAs to Dallas Microlan(tm) Devices.
  It currently supports DS1820/DS18S20 Digital Thermometers
  Copyright (C) 2000 Christof Klaiber

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
  PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  Place, Suite 330, Boston, MA  02111-1307  USA


  Christof Klaiber <christof.klaiber@gmx.de>
  Lachnerstr. 22
  76131 Karlsruhe
  Germany
*/
#include <PalmOS.h>
#include <SerialMgrOld.h>

#include "TempRsc.h"
#include "Temp.h"
#include "ds1820.h"
#include "onewire.h"

/* CRC table for Dallas 8-bit CRC calculations from App Note #27 */
UInt8 crc_table[] = {
     0, 94,188,226, 97, 63,221,131,194,156,126, 32,163,253, 31, 65,
   157,195, 33,127,252,162, 64, 30, 95,  1,227,189, 62, 96,130,220,
    35,125,159,193, 66, 28,254,160,225,191, 93,  3,128,222, 60, 98,
   190,224,  2, 92,223,129, 99, 61,124, 34,192,158, 29, 67,161,255,
    70, 24,250,164, 39,121,155,197,132,218, 56,102,229,187, 89,  7,
   219,133,103, 57,186,228,  6, 88, 25, 71,165,251,120, 38,196,154,
   101, 59,217,135,  4, 90,184,230,167,249, 27, 69,198,152,122, 36,
   248,166, 68, 26,153,199, 37,123, 58,100,134,216, 91,  5,231,185,
   140,210, 48,110,237,179, 81, 15, 78, 16,242,172, 47,113,147,205,
    17, 79,173,243,112, 46,204,146,211,141,111, 49,178,236, 14, 80,
   175,241, 19, 77,206,144,114, 44,109, 51,209,143, 12, 82,176,238,
    50,108,142,208, 83, 13,239,177,240,174, 76, 18,145,207, 45,115,
   202,148,118, 40,171,245, 23, 73,  8, 86,180,234,105, 55,213,139,
    87,  9,235,181, 54,104,138,212,149,203, 41,119,244,170, 72, 22,
   233,183, 85, 11,136,214, 52,106, 43,117,151,201, 74, 20,246,168,
   116, 42,200,150, 21, 75,169,247,182,232, 10, 84,215,137,107, 53
};

/*
	Name:		MatchROM
	Purpose:	Send/ a Match Rom command to the indicated 1-wire device
	Input:		UInt16	timeout	Timeout-Value in Ticks
			UInt16	rom	Number of Rom to search for
	Result:		Int8	0 if correct, -1 if error
			
	Name: Christof Klaiber <christof.klaiber@gmx.de>
	Versions:	25.01.2001	First working Release
			04.03.2001	adapted to working SearchRom
			
	Remarks:	not tested
	This function is taken from digitemp-1.3/onewire.c by Brian C. Lane <bcl@brianlane.com>
*/
Int8	MatchROM(UInt16 timeout, UInt16 rom) {
	UInt8	x;
	UInt8* 	myroms;
	if (TouchReset(timeout) < 0) {
		FrmAlert(InitializingAlert);
		return -1;
	}

	if (TouchByte(timeout, 0x55) < 0) {
		FrmAlert(MatchRomAlert);
		return -1;
	}
	
	myroms = MemHandleLock(rom_list.roms);
	for ( x = 0; x < 8; x++) {
		if (TouchByte(timeout, myroms[(rom*8)+x]) < 0) {
			FrmAlert(WritingByteAlert);
			return -1;
		}
		
	}
	MemHandleUnlock(rom_list.roms);
	return 0;
}

/*
	Name:		ReadScratchpad
	Purpose:	Send/ a Match Rom command to the indicated 1-wire device
	Input:		UInt16	timeout	Timeout-Value in Ticks
			UInt8*	scpad	Pointer to 9 Bytes for SP-Data
	Result:		Int8	0 if correct, -1 if error
			
	Name: Christof Klaiber <christof.klaiber@gmx.de>
	Versions:	25.01.2001	First working Release
	 
	This function is taken from digitemp-1.3/onewire.c by Brian C. Lane <bcl@brianlane.com>
*/
Int8 ReadScratchpad(UInt16 timeout, UInt8 *scpad) {
	Int16	byte;
	UInt8	x;
	UInt8	crc;
	UInt8	exOrResult;
	
	if (TouchByte(timeout, 0xbe) < 0) {
		FrmAlert(ReadScratchpadAlert1);
		return -1;
	}
	
	for(x = 0; x < 9; x++) {
		if ((byte = TouchByte(timeout, 0xff)) < 0) {
			FrmAlert(ReadScratchpadAlert2);
			return -1;
		}
		scpad[x] = (UInt8) byte;
	}
	
	if (TouchReset(timeout) < 0) {
		FrmAlert(InitializingAlert);
		return -1;
	}
	
	crc = 0;
	
	for(x = 0; x < 9; x++)
	{
		exOrResult = crc ^ scpad[x];
		crc = crc_table[exOrResult];
	}
	
	if(crc != 0)
	{
		FrmAlert(CRCAlert);
		return -1;
	}
	
	return 0;
}

/*
	Name:		WriteLimits
	Purpose:	Write Temperature high and low limits
	Input:		UInt16	timeout	Timeout-Value in Ticks
			UInt16	tl	low Temperature Value
			UInt16	th	high Temperature Value
	Result:		Int8	0 if correct, -1 if error
			
	Name: Christof Klaiber <christof.klaiber@gmx.de>
	Versions:	25.01.2001	First working Release
	 
	This function is taken from digitemp-1.3/onewire.c by Brian C. Lane <bcl@brianlane.com>
*/
Int8	WriteLimits(UInt16 timeout, UInt16 tl, UInt16 th) {
	
	if (TouchByte(timeout, 0x4e) < 0) {
		FrmAlert(WriteScratchpadAlert);
		return -1;
	}
	
	if (TouchByte(timeout, th) < 0) {
		FrmAlert(WriteLimitsAlert);
		return -1;
	}
	
	if (TouchByte(timeout, tl) < 0) {
		FrmAlert(WriteLimitsAlert);
		return -1;
	}
	if (TouchReset(timeout) < 0) {
		FrmAlert(InitializingAlert);
		return -1;
	}
	
	return 0;
}


/*
	Name:		WriteEE
	Purpose:	Write the limits to EEPROM
	Input:		UInt16	fail_time	Timeout
			UInt16	read_time	Timeout move data in DS1820
	Result:		Int8	0 if correct, -1 if error

	Comment:	Not sure about if it will work with the simple adapter
			
	Name: Christof Klaiber <christof.klaiber@gmx.de>
	Versions:	25.01.2001	First working Release
	
	This function is taken from digitemp-1.3/onewire.c by Brian C. Lane <bcl@brianlane.com>
*/
Int8 WriteEE(UInt16 fail_time, UInt16 read_time) {
	if (TouchByte(fail_time, 0x48) < 0) {
		FrmAlert(WritingLimitsEEAlert);
		return -1;
	}
	
	SysTaskDelay(read_time);

	if (TouchReset(fail_time) < 0) {
		FrmAlert(InitializingAlert);
		return -1;
	}
	return 0;
}
/*
	Name:		ReadEE
	Purpose:	Copy the EEPROM to the scratchpad
	Input:		UInt16	timeout	Timeout-Value in Ticks
	Result:		Int8	0 if correct, -1 if error
			
	Name: Christof Klaiber <christof.klaiber@gmx.de>
	Versions:	25.01.2001	First working Release
	 
	This function is taken from digitemp-1.3/onewire.c by Brian C. Lane <bcl@brianlane.com>
*/
Int8	ReadEE(UInt16 timeout) {
	if(TouchByte(timeout, 0xb8) < 0)
	{
		FrmAlert(ReadingLimitsEEAlert);
		return -1;
	}
	
	if (TouchReset(timeout) < 0) {
		FrmAlert(InitializingAlert);
		return -1;
	}
	return 0;
}

/*
	Name:		ReadPWR
	Purpose:	Read the power status of the device
	Input:		UInt16	timeout	Timeout-Value in Ticks
	Result:		Int8	Power Status
				-1 if error
			
	Name: Christof Klaiber <christof.klaiber@gmx.de>
	Versions:	25.01.2001	First working Release
	 
	This function is taken from digitemp-1.3/onewire.c by Brian C. Lane <bcl@brianlane.com>
*/
Int16	ReadPWR(UInt16 timeout) {
	Int16 p;
	
	if (TouchByte(timeout, 0xb4) < 0)
	{
		FrmAlert(ReadingPowerAlert);
		return -1;
	}
	
	p = TouchBits(timeout, 1, 0xff);

	if (p < 0)
	{
		FrmAlert(ReadingPowerAlert);
		return -1;
	}

	if (TouchReset(timeout) < 0) {
		FrmAlert(InitializingAlert);
		return -1;
	}
	
	return p;
}

/*
	Name:		ReadTemperature
	Purpose:	Do the Temperature Conversation
	Input:		UInt16	fail_time	Timeout-Value in Ticks
			UInt16	read_time	Timeout-Value for Conversation
	Result:		Int8	0 if correct, -1 if error
			
	Name: Christof Klaiber <christof.klaiber@gmx.de>
	Versions:	25.01.2001	First working Release
	 
	This function is taken from digitemp-1.3/onewire.c by Brian C. Lane <bcl@brianlane.com>
*/
Int8 ReadTemperature(UInt16 fail_time, UInt16 read_time) {

	if (TouchByte(fail_time, 0x44) < 0)
	{
		FrmAlert(ReadTemperatureAlert);
		return -1;
	}
	SysTaskDelay(read_time);
	
	if (TouchReset(fail_time) < 0) {
		FrmAlert(InitializingAlert);
		return -1;
	}
	
	return 0;	
}

/*
	Name:		SkipROM
	Purpose:	Skip the MatchROM for single devices
	Input:		UInt16	timeout	Timeout-Value in Ticks
	Result:		Int8	0 if correct, -1 if error
			
	Name: Christof Klaiber <christof.klaiber@gmx.de>
	Versions:	25.01.2001	First working Release
	 		04.03.2001	deleted redundant TouchReset
			
	This function is taken from digitemp-1.3/onewire.c by Brian C. Lane <bcl@brianlane.com>
*/
Int8 SkipROM(UInt16 timeout) {
	if(TouchReset(timeout) < 0) {
		FrmAlert(InitializingAlert);
		return -1;
	}
	
	if(TouchByte(timeout, 0xcc) < 0) {
		FrmAlert(SkipRomAlert);
		return -1;
	}

	return 0;
}

/*
	Name:		ReadROM
	Purpose:	Get the ROM content for a single device
	Input:		UInt16	timeout	Timeout-Value in Ticks
	Result:		Int8	0 if correct, -1 if error
			
	Name: Christof Klaiber <christof.klaiber@gmx.de>
	Versions:	25.01.2001	First working Release
	 
	This function is taken from digitemp-1.3/onewire.c by Brian C. Lane <bcl@brianlane.com>
*/
Int8 ReadROM(UInt16 timeout) {
	UInt16	x;
	Char	ROM[32];
	
	if (TouchReset(timeout) < 0) {
		FrmAlert(InitializingAlert);
		return -1;
	}
	
	if (TouchByte(timeout, 0x33) < 0) {
		FrmAlert(ReadROMAlert);
		return -1;
	}

	
	for(x = 0; x < 8; x++)
	{
		StrIToA(ROM, TouchByte(timeout, 0xff));
		WinDrawChars(ROM, StrLen(ROM), 5, x * 10 + 30);
	}
	return 0;
}

/*
	Name:		pow
	Purpose:	Get the ROM content for a single device
	Input:		UInt8	man Mantisse
			UInt8	exp Exponent
	Result:		UInt8	man^exp
			
	Name: Christof Klaiber <christof.klaiber@gmx.de>
	Versions:	26.01.2001	First working Release
*/

UInt8 pow(UInt8 man, UInt8 exp) {
	UInt32 i;
	UInt32 res = 1;

	for (i = 0; i < exp; i++)
		res *= man;
	return res;
};

/*
	Name:		SearchRom
	Purpose:	Find the 64 bit serial numbers of all attached devices
	Input:		timeoute Timeout-Value in Ticks
	Result:		number of devices found
			-1: error
	Globals:	struct roms	rom_list	structure to store serial numbers
			
	Name: Christof Klaiber <christof.klaiber@gmx.de>
	Versions:	26.01.2001	First working Release
			04.04.2001	Tests and bug fixes
			06.04.2001	Small bugfix with crc and no sensor found
	This function is taken from digitemp-1.3/onewire.c by Brian C. Lane <bcl@brianlane.com>
*/

UInt32 SearchROM(UInt16 timeout)
{
	UInt8	crc, shift_reg[64], serial_num[64], rom_buf[8];
	UInt8	bits, serial_idx, done, x, byte;
	UInt8*	myroms;
	
	serial_idx = 0;
	
	for(x = 0; x < 64; x++) {
		shift_reg[x] = 1;
	}
	
	for(x = 0; x < 8; x++) {
		rom_buf[x] = 0;
	}
	
	if(rom_list.roms != NULL)
		MemHandleResize(rom_list.roms, 1);
	else
		rom_list.roms = MemHandleNew(1);

	rom_list.max = 0;
	
	done = 0;
	while(!done) {
		if (TouchReset(timeout) < 0) {
			return -1;
			
		}
		
		TouchByte(timeout, 0xf0);
		
		for(x = 0; x < 64; x++)
		{
			bits = TouchBits(timeout, 2, 0xff);
			switch(bits & 0xc0) {
			case 0x00:
				if (shift_reg[x] == 1) {
					TouchBits(timeout, 1, 0xff);
					shift_reg[x] = 1;
					serial_num[x] = 1;
				} else {
					TouchBits(timeout, 1, 0x00);
					shift_reg[x] = 0;
					serial_num[x] = 0;
				}
				break;
			case 0x40:	
				TouchBits(timeout, 1, 0xff);
				shift_reg[x] = 0;
				serial_num[x] = 1;
				break;
			case 0x80:
				TouchBits(timeout, 1, 0x00);
				shift_reg[x] = 0;
				serial_num[x] = 0;
				break;
			case 0xc0:
				return -1;
				break;
			
			default:
				break;
			
			
			} /* switch */
		} /* for */
		byte = 0;
		for(x = 0; x < 64; x++) 
		{
			if (serial_num[x] == 1)
				byte += pow(2, (x % 8));

			if ((x % 8) == 7) {
				rom_buf[x / 8] = byte;
				byte = 0;
			}
		}
		
		crc = 0;
		
		for(x = 0; x < 8; x++) {
			crc = crc_table[crc ^ rom_buf[x]];
		}
		
		if(crc == 0x00) {
			if ((rom_buf[0] == DS1820_FAMILY) || (rom_buf[0] == DS1822_FAMILY) || (rom_buf[0] == DS18B20_FAMILY)) {
				rom_list.max++;
				if (MemHandleResize(rom_list.roms, rom_list.max * 8)) {
					FrmAlert(MemoryAlert);
					return -1;
				}
				
				myroms = MemHandleLock(rom_list.roms);
				for(x = 0; x < 8; x++) {
					myroms[serial_idx*8 + x] = rom_buf[x];
				}
				serial_idx++;
				MemHandleUnlock(rom_list.roms);
			 
			} else {
				FrmAlert(CRCAlert);
			}
		}
		done = 1;
		for(x = 0; x < 64; x++) {
			if (shift_reg[63-x] == 0) {
				shift_reg[63-x] = 1;
			} else {
				shift_reg[63-x] = 0;
				done = 0;
				break;
			}
		}
	}
	if (TouchReset(timeout) < 0) {
		return -1;
		
	}
	return serial_idx;
}
